--Licensed Materials - Property of IBM
--(C) Copyright IBM Corp. 2012 All Rights Reserved
--US Government Users Restricted Rights - Use, duplicate or
--disclosure restricted by GSA ADP Schedule Contract with
--IBM Corp.
--
--This script is meant to create the schema for sFlow
--This script will be run using smreset OR cfgdbcmd after 
--installing sdnm feature on director server
--During installation/upgrade,this file will be copied to proddata/dbconfig/sql/db2/external/ on director server
CREATE TABLE Sflow_DatagramHeader(
  Id 					BIGINT 		NOT NULL GENERATED BY DEFAULT AS IDENTITY,
  SflowVersion 			SMALLINT 	NOT NULL,
  AgentIPAddress		BIGINT,
  AgentIPAddressStr		VARCHAR(46)	NOT NULL,
  AgentIPAddressVersion	SMALLINT 	NOT NULL,
  SubAgentId 			INTEGER 	NOT NULL,
  SequenceNumber 		BIGINT 		NOT NULL,
  SysUpTime 			BIGINT 		NOT NULL,
  SampleReceivedTime 	TIMESTAMP 	NOT NULL 	DEFAULT CURRENT_TIMESTAMP,
  SflowDatagram 		BLOB,
  PRIMARY KEY (Id)
);

CREATE TABLE Sflow_FlowSample (
  Id 					BIGINT 		NOT NULL GENERATED BY DEFAULT AS IDENTITY,
  DatagramHeaderId 		BIGINT 		NOT NULL,
  SequenceNumber 		BIGINT 		NOT NULL,
  DataSourceType 		SMALLINT 	NOT NULL,
  DatasourceId 			INTEGER 	NOT NULL,
  DataSourceOID 		VARCHAR(128),
  SamplingRate 			INTEGER 	NOT NULL,
  SamplePool 			INTEGER 	NOT NULL,
  PacketDrops 			INTEGER 	NOT NULL,
  InputInterface 		INTEGER 	NOT NULL,
  OutputInterface 		INTEGER 	NOT NULL,
  InputInterfaceOID 	VARCHAR(128) NOT NULL,
  OutputInterfaceOID 	VARCHAR(128) NOT NULL,
  PRIMARY KEY (Id)
);

CREATE TABLE Sflow_FlowRecord (
  Id 					BIGINT 		NOT NULL GENERATED BY DEFAULT AS IDENTITY,
  FlowSampleId 			BIGINT 		NOT NULL,
  DataFormat 			INTEGER 	NOT NULL,
  HeaderProtocol 		INTEGER 	NOT NULL,
  FrameLength 			INTEGER 	NOT NULL,
  StrippedOctets 		INTEGER 	NOT NULL,
  MacPacketLength 		INTEGER,
  SourceMacStr 			VARCHAR(20) NOT NULL,
  DestinationMacStr 	VARCHAR(20) NOT NULL,
  SourceMac 			BIGINT 		NOT NULL ,
  DestinationMac 		BIGINT 		NOT NULL,
  EtherentPacketType 	INTEGER,
  SourceIP 				BIGINT,
  DestinationIP 		BIGINT,
  SourceIPStr 			VARCHAR(46),
  DestinationIPStr 		VARCHAR(46),
  IPVersion 			SMALLINT,
  ServiceType 			INTEGER,
  IPPacketLength 		BIGINT, 
  Identification 		INTEGER, 
  Flags 				INTEGER,
  FragmentOffset 		INTEGER,
  TimeToLive 			SMALLINT,
  DataProtocol 			SMALLINT,
  TrafficClass 			SMALLINT,
  FlowLabel 			INTEGER,
  NextHeader 			INTEGER,   
  SourcePort 			INTEGER,
  DestinationPort 		INTEGER,
  Priority 				INTEGER,
  CanonicalFormatIndicator INTEGER,
  VLANID 				INTEGER,
  PRIMARY KEY (Id)
);

CREATE INDEX I_Sflow_ReceivedTime ON Sflow_DatagramHeader (SampleReceivedTime);
CREATE INDEX I_Sflow_InputInterface ON Sflow_FlowSample (InputInterface);
CREATE INDEX I_Sflow_OutputInterface ON Sflow_FlowSample (OutputInterface);
CREATE INDEX I_Sflow_InputInterfaceOID ON Sflow_FlowSample (InputInterfaceOID);
CREATE INDEX I_Sflow_OutputInterfaceOID ON Sflow_FlowSample (OutputInterfaceOID);
CREATE INDEX I_Sflow_SourceMac ON Sflow_FlowRecord (SourceMac);
CREATE INDEX I_Sflow_DestinationMac ON Sflow_FlowRecord (DestinationMac);
CREATE INDEX I_Sflow_SourceIP ON Sflow_FlowRecord (SourceIP);
CREATE INDEX I_Sflow_DestinationIP ON Sflow_FlowRecord (DestinationIP);
CREATE INDEX I_Sflow_VLANID ON Sflow_FlowRecord (VLANID);

CREATE  VIEW  Sflow_FlowRecordView
AS  SELECT 
		sfh.SflowVersion,
		sfh.AgentIPAddress,
		sfh.AgentIPAddressStr,
		sfh.SubAgentId, 
		sfh.SequenceNumber AS FhSequenceNumber,
		sfh.SysUpTime, 
		sfh.SampleReceivedTime, 
		fs.DatagramHeaderId,
		fs.SequenceNumber AS FsSequenceNumber, 
		fs.DataSourceType,
		fs.DatasourceId, 
		fs.DataSourceOID,
		fs.SamplingRate,
		fs.SamplePool, 
		fs.PacketDrops, 
		fs.InputInterface,
		fs.OutputInterface,
		fs.InputInterfaceOID,
		fs.OutputInterfaceOID, 
		fr.Id AS FlowRecordId, 
		fr.FlowSampleId,
		fr.DataFormat, 
		fr.HeaderProtocol, 
		fr.FrameLength,
		fr.StrippedOctets, 
		fr.MacPacketLength, 
		fr.SourceMac,
		fr.DestinationMac,
		fr.SourceMacStr,
		fr.DestinationMacStr, 
		fr.EtherentPacketType,
		fr.SourceIP,
		fr.DestinationIP, 
		fr.SourceIPStr,
		fr.DestinationIPStr,
		fr.IPVersion, 
		fr.ServiceType,
		fr.IPPacketLength, 
		fr.Identification, 
		fr.Flags,
		fr.FragmentOffset, 
		fr.TimeToLive, 
		fr.DataProtocol,
		fr.TrafficClass, 
		fr.FlowLabel, 
		fr.NextHeader,
		fr.SourcePort,
		fr.DestinationPort,
		fr.Priority, 
		fr.CanonicalFormatIndicator, 
		fr.VLANID
FROM 	Sflow_DatagramHeader sfh, 
		Sflow_FlowSample fs, 
		Sflow_FlowRecord fr
WHERE 	sfh.id = fs.DatagramHeaderId AND 
		fs.id = fr.FlowSampleId;


COMMENT ON Sflow_DatagramHeader (
  Id 				IS  'Primary Key for Sflow_DatagramHeader',
  SflowVersion 		IS 'Sflow version. Currently its V5',
  AgentIPAddress	IS	'Agent OR device IP Address',
  AgentIPAddressStr	IS	'Agent OR device IP Address in String format',
  AgentIPAddressVersion	IS 	'Agent OR device IP Address version 4=IPv4, 6=IPv6',
  SubAgentId 		IS 'Used to distinguish between datagram streams from separate agent sub entities within an device.',
  SequenceNumber 	IS 'Incremented with each sample datagram generated by a sub-agent within an agent. ',
  SysUpTime 		IS 'Current time (in milliseconds since device last booted. Should be set as close to datagram transmission time as possible.',
  SampleReceivedTime IS 'The time at which the sample is received by collector',
  SflowDatagram 	IS'Actual datagram sent by sFlow agent'
);

COMMENT ON Sflow_FlowSample (
  Id 				IS 'Primary Key for Sflow_FlowSample table',
  DatagramHeaderId 	IS 'Foreign Key for SflowDatagram Table',
  SequenceNumber 	IS 'Incremented with each flow sample generated by this source_id. Note: If the agent resets the sample_pool then it must also reset the sequence_number.',
  DataSourceType 	IS 'Data source Type ',
  DatasourceId 		IS 'Data source Id  on device  for  which the sample is collected',
  DataSourceOID 	IS 'Data source OID on device for which the sample is collected',
  SamplingRate 		IS 'Fow sampling rate applied for the data source',
  SamplePool 		IS 'Total number of packets that could have been sampled (i.e. packets skipped by sampling process + total number of samples) ',
  PacketDrops 		IS 'Number of times that the sFlow agent detected that a packet marked to be sampled was dropped due to lack of resources.',
  InputInterface 	IS 'Interface on which packet was received on. ',
  OutputInterface 	IS 'Interface on which packet was sent on. ',
  InputInterfaceOID IS 'OID of an interface on which packet was received on.',
  OutputInterfaceOID IS 'OID of an interface on which packet was sent to.'
);

COMMENT ON Sflow_FlowRecord (
  Id 				IS 'Primary Key for Sflow_FlowRecord',
  FlowSampleId 		IS 'Foreign Key for FlowSample Table',
  DataFormat 		IS 'Data format of Flow records, 1=Raw Packet Header, 2=Ethernet Frame Data,3=IPv4,4=IPv6..etc',
  HeaderProtocol 	IS 'Protocol of Raw Packet Header. 1=ethernet, .... 11=IPv4, 12=IPv6 ',
  FrameLength 		IS 'Original length of packet before sampling.',
  StrippedOctets 	IS 'The number of octets removed from the packet before extracting the header octets.',
  MacPacketLength 	IS 'The length of the MAC packet received on the network, excluding lower layer encapsulations and framing bits but including FCS octets ',
  SourceMacStr 		IS'Formatted Source MAC address ',
  DestinationMacStr IS'Formatted Destination MAC address',
  SourceMac 		IS 'Source MAC address as integer type',
  DestinationMac 	IS 'Destination MAC address as integer type',
  EtherentPacketType IS 'Ethernet packet type: 2048=IPv4, 34525=IPv6, 33024=VLAN tagged packet, 32864=ARP',
  SourceIP 			IS 'Source IP Address in Integer,Applicable for IPv6/IPV4',
  DestinationIP 	IS 'Destination IP address in Integer,Applicable for IPv6/IPV4',
  SourceIPStr 		IS 'Formatted Source IP Address ,Applicable for IPv6/IPV4',
  DestinationIPStr 	IS 'Formatted Destination IP address,Applicable for IPv6/IPV4',
  IPVersion 		IS '4=(IPv4),6=(IPv6)',
  ServiceType 		IS 'The Type of Service provides an indication of the abstract parameters of the quality of service desired.Applicable for IPV4',
  IPPacketLength 	IS 'Total Packet length ,Applicable for both IPv6/IPV4',
  Identification 	IS 'An identifying value assigned by the sender to aid in assembling the fragments of a datagram. Applicable for IPV4',
  Flags 			IS 'Various Control Flags. Applicable for IPV4',
  FragmentOffset 	IS 'This field indicates where in the datagram this fragment belongs. The fragment offset is measured in units of 8 octets (64 bits). The first fragment has offset zero. Applicable for IPV4',
  TimeToLive 		IS 'This field indicates the maximum time the datagram is allowed to remain in the internet system. Applicable for IPV4',
  DataProtocol 		IS 'This field indicates the next level protocol used in the data portion of the internet datagram.  6=TCP, 17=UDP,Applicable for IPV4',
  TrafficClass 		IS 'Applicable for IPv6',
  FlowLabel 		IS 'Label for a data flow. Applicable for IPv6',
  NextHeader 		IS 'Identifies the type of header immediately following the IPv6 header, Applicable for IPv6',
  SourcePort 		IS 'L3 Source port number',
  DestinationPort 	IS 'L3 Destination port number',
  Priority 			IS 'Defines user priority, Applicable for VLAN tagged Packet',
  CanonicalFormatIndicator IS 'Canonical Format Indicator, Applicable for VLAN tagged Packet',
  VLANID 			IS 'VALN Id,Applicable for VLAN tagged Packet'
);
